unit Leech;

{This is the "Part II" external; the user interface external, which plays the actual game}

interface
	uses
		HermHeaders;

	procedure main (HermSetup: UserXIPtr);

implementation

	const
		data_path = 'Externals:Leech Folder:';	{pathname to data files}
		pause_string = '[PAUSE]';					{string displayed when pausing}
		vers_num = '1.02';							{version number of this release}
		main_menu_stage = 7;						{stage that prompts for main menu command}
		lll_menu_stage = 19;						{stage that prompts for an LLL BBS menu command}
		fight_stage = 52;							{stage that begins a fight}
		pause_stage = 99;							{stage for pausing after a print_text_rsrc}
		exit_stage = 100;							{stage that exits the external application}
		opening_txt = 128;							{ID of the Opening text rsrc}
		board_list_txt = 129;						{ID of the Board List text rsrc}
		main_menu_txt = 130;						{ID of the Main Menu text rsrc}
		LLL_menu_txt = 131;						{ID of the LLL Menu text rsrc}
		backup_list_txt = 132;						{ID of the Backup List text rsrc}
		software_list_txt = 133;					{ID of the Software List text rsrc}
		final_battle_txt = 134;						{ID of the Final Battle text rsrc}
		credits_txt = 135;							{ID of the Credits text rsrc}
		won_final_battle_txt = 136;				{ID of the Won Final Battle text rsrc}
		board_data_txt = 137;						{ID of the Board Data text rsrc}
		eof_char = '';								{char signifying the end of the data text rsrc}
		text_type = 'TEXT';							{type for resources and text files}
		data_type = 'DATA';						{type for data files}
		option_type = 'LCOP';						{type for leech options resource}
		creator_type = 'HRMS';					{creator for all files}
		bulletin_fName = 'Bulletin';				{file name for the Bulletin text file}
		hall_fName = 'Hall of Fame';				{file name for the Hall of Fame text file}
		help_fName = 'Leech Help';					{file name for the Leech Help text file}
		leech_data_fName = 'Leech Users Data';	{file name for the Leech Users Data data file}
		divide_line = '._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._._.';	{line for text files}
		garbage_string = '@$#%^';				{garbage string used in initiating searches}
		inval_leech = 'Invalid leech.';				{reported several times; is a constant to save space}
		not_enough_money = 'That price is over your savings!';	{""}
		are_you_sure = 'Are you sure you want to ';				{""}

	function cstr (i: integer): string;				{convert integer to string}
		var
			t: str255;
	begin
		NumToString(i, t);
		cstr := t
	end;

	function cstrl (i: longint): string;				{convert longint to string}
		var
			t: str255;
	begin
		numtostring(i, t);
		cstrl := t
	end;

	function s2n (the_string: str255): integer;		{convert string to integer}
		var
			temp: longint;
	begin
		StringToNum(the_string, temp);
		s2n := temp
	end;

	function ucase (the_string: string): string;	{convert lowercase string to uppercase string}
		var
			i: integer;
	begin
		for i := 1 to length(the_string) do
			if the_string[i] in ['a'..'z'] then
				the_string[i] := chr(ord(the_string[i]) - ord('a') + ord('A'));
		ucase := the_string
	end;

	function space_it (it: string; how_long: integer): string;	{make sure a string is a certain length, appending spaces if it isn't}
		var
			temp: string;
			i: integer;
	begin
		temp := it;
		if length(it) < how_long then
			for i := (length(it) + 1) to how_long do
				temp := concat(temp, ' ');
		space_it := temp
	end;

	procedure prt (the_string: string; var HermSetup: UserXIPtr);
  {used to cut down on code size, since the XHRM resource cannot be > 32K}
	begin
		OutLine(the_string, true, 0, HermSetup^.procs[1])
	end;

	procedure parseerr (err: OSErr; var HermSetup: UserXIPtr);	{check for an error}
	begin
		if err <> noerr then
			with HermSetup^ do
				prt(concat('Error #', cstr(err), ' in stage #', cstr(privates^^.stage[curNode^])), HermSetup)
	end;

	function file_size (the_path: str255; var HermSetup: UserXIPtr): longint;	{returns the size of a file in bytes}
		var
			ref_num: integer;
			err: OSErr;
			the_longint: longint;
	begin
		err := FSOpen(the_path, 0, ref_num);
		parseerr(err, HermSetup);
		err := GetEOF(ref_num, the_longint);
		parseerr(err, HermSetup);
		file_size := the_longint;
		err := FSClose(ref_num);
		parseerr(err, HermSetup)
	end;

	function file_exists (fName: str255): boolean;			{returns whether the file exists on the desktop}
		var
			err: OSErr;
			info: FInfo;
	begin
		err := GetFInfo(fName, 0, info);
		file_exists := err <> fnfErr
	end;

	function rnd (high: integer; var HermSetup: UserXIPtr): integer;	{pseudo-random, returns a number from 0 to high}
		var
			temp: integer;
	begin
		with HermSetup^.privates^^ do
			repeat
				seed := (29 * seed + 1) mod 1024;
				temp := abs(trunc((seed / 1024) * (high + 1)))
			until (temp <= high) and (temp >= 0);
		rnd := temp
	end;

	procedure read_option_rec (var the_rec: leech_option_rec);
		var
			option_hand: leechOptionHand;
	begin
		option_hand := leechOptionHand(GetResource(option_type, 100));
		the_rec := option_hand^^;
		ReleaseResource(handle(option_hand))
	end;

	procedure find_user (the_number, other_number: integer; var HermSetup: UserXIPtr; var the_leech: leech_rec);
  {search for a leech with real user number the_number in the data file}
		var
			err: OSErr;
			ref_num: integer;
			the_pos, the_size, the_longint: longint;
	begin
		with HermSetup^ do
			begin
				privates^^.successful := file_exists(concat(filesPath^, data_path, leech_data_fName)) and (the_number <> other_number);
				if privates^^.successful then
					begin
						err := FSOpen(concat(filesPath^, data_path, leech_data_fName), 0, ref_num);
						parseerr(err, HermSetup);
						the_pos := 0;
						err := GetEOF(ref_num, the_size);
						parseerr(err, HermSetup);
						the_longint := SizeOf(leech_rec);
						the_leech.RealNumber := -1;
						while (the_pos < the_size) and (the_number <> the_leech.RealNumber) do
							begin
								err := FSRead(ref_num, the_longint, @the_leech);
								parseerr(err, HermSetup);
								the_pos := the_pos + the_longint
							end;
						err := FSClose(ref_num);
						parseerr(err, HermSetup);
						privates^^.successful := the_number = the_leech.RealNumber
					end
			end
	end;

	procedure read_usernum (which: integer; var HermSetup: UserXIPtr; var the_leech: leech_rec; var ref_num: integer);
  {read a certain number leech from the leech data file}
		var
			err: OSErr;
			the_longint: longint;
	begin
		with HermSetup^ do
			begin
				err := SetFPos(ref_num, fsFromStart, longint(which - 1) * SizeOf(leech_rec));
				parseerr(err, HermSetup);
				the_longint := SizeOf(leech_rec);
				err := FSRead(ref_num, the_longint, @the_leech);
				parseerr(err, HermSetup)
			end
	end;

	procedure extract_line (var the_loc: integer; var the_text: CharsHandle; var temp: str255; var finished: boolean);
	begin
		temp := '';
		the_loc := the_loc + 1;
		while not (the_text^^[the_loc] in [chr(13), eof_char]) do
			begin
				temp := concat(temp, the_text^^[the_loc]);
				the_loc := the_loc + 1
			end;
		finished := the_text^^[the_loc] = eof_char
	end;

	procedure print_text_rsrc (ID: integer; var HermSetup: UserXIPtr);
  {print a 'TEXT' resource via OutLine's}
		var
			the_text: handle;
	begin
		the_text := GetResource(text_type, ID);
		parseerr(ResError, HermSetup);
		DetachResource(the_text);
		MoveHHi(the_text);
		HLock(the_text);
		with HermSetup^.n[HermSetup^.curNode^]^ do
			begin
				textHnd := TextHand(the_text);
				OpenTextSize := GetHandleSize(handle(textHnd));
				curTextPos := 0;
				BoardAction := ListText
			end
	end;

	procedure write_halloffame (var HermSetup: UserXIPtr);	{display the Hall of Fame}
	begin
		with HermSetup^ do
			begin
				bCR(procs[0]);
				prt('These are the best leeches ever!', HermSetup);
				prt('--------------------------------', HermSetup);
				if file_size(concat(filesPath^, data_path, hall_fName), HermSetup) <= 0 then
					prt('None are!', HermSetup)
				else
					begin
						bCR(procs[0]);
						if ReadTextFile(concat(data_path, hall_fName), 0, true, procs[10]) then
							n[curNode^]^.BoardAction := ListText
					end
			end
	end;

	procedure write_user (var the_leech: leech_rec; var HermSetup: UserXIPtr);	{write a leech to the leech data file}
		var
			ref_num: integer;
			the_longint: longint;
			err: OSErr;
	begin
		with HermSetup^ do
			begin
				if not file_exists(concat(filesPath^, data_path, leech_data_fName)) then
					begin
						err := Create(concat(filesPath^, data_path, leech_data_fName), 0, creator_type, data_type);
						parseerr(err, HermSetup)
					end;
				err := FSOpen(concat(filesPath^, data_path, leech_data_fName), 0, ref_num);
				parseerr(err, HermSetup);
				if not the_leech.in_file then
					begin
						err := GetEOF(ref_num, the_longint);
						parseerr(err, HermSetup);
						err := SetEOF(ref_num, the_longint + SizeOf(leech_rec));
						parseerr(err, HermSetup)
					end;
				err := SetFPos(ref_num, fsFromStart, SizeOf(leech_rec) * longint(the_leech.Number - 1));
				parseerr(err, HermSetup);
				the_longint := SizeOf(leech_rec);
				the_leech.in_file := true;
				err := FSWrite(ref_num, the_longint, @the_leech);
				parseerr(err, HermSetup);
				err := FSClose(ref_num);
				parseerr(err, HermSetup)
			end
	end;

	procedure read_user (var HermSetup: UserXIPtr);
  {read a leech from the leech data file}
		var
			err: OSErr;
			ref_num: integer;
			the_longint, file_pos, file_eof: longint;
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					bCR(procs[0]);
					find_user(n[curNode^]^.thisUser.UserNum, 0, HermSetup, the_leech[curNode^]);
					if successful then
						prt(concat('Glad to see you back, ', the_leech[curNode^].Name, '!'), HermSetup)
					else
						with privates^^.the_leech[curNode^] do
							begin
								Name := n[curNode^]^.thisUser.UserName;
								if file_exists(concat(filesPath^, data_path, leech_data_fName)) then
									Number := file_size(concat(filesPath^, data_path, leech_data_fName), HermSetup) div SizeOf(leech_rec) + 1
								else
									Number := 1;
								RealNumber := n[curNode^]^.thisUser.UserNum;
								Level := 1;
								Software := 'Dumb Terminal';
								SoftwarePower := 1;
								Backup := 'None';
								BackupPower := 1;
								Money := rnd(200, HermSetup);
								MegsMax := 10;
								DrivesToCrash := 10;
								Bank := 0;
								GetDateTime(the_longint);
								IUDateString(the_longint, shortDate, DateCalled);
								TimesOnToday := 0;
								FastBacks := 0;
								Trojans := 0;
								CallsToday := 10;
								HacksToday := 3;
								CurMegs := 0;
								in_file := false;
								write_user(the_leech[curNode^], HermSetup);
								prt('Hello, new leech!', HermSetup)
							end
				end
	end;

	procedure check_user (var HermSetup: UserXIPtr);	{check leech to make sure he can play this time}
		var
			the_longint: longint;
			temp: str255;
	begin
		with HermSetup^ do
			with privates^^.the_leech[curNode^] do
				begin
					GetDateTime(the_longint);
					IUDateString(the_longint, shortDate, temp);
					if DateCalled = temp then
						TimesOnToday := TimesOnToday + 1
					else
						begin
							DateCalled := temp;
							TimesOnToday := 1
						end;
					CurMegs := MegsMax;
					CallsToday := 10;
					HacksToday := 3;
					FastBacks := FastBacks + 1;
					Trojans := Trojans + 1;
					if (TimesOnToday > privates^^.the_options.plays_allowed) and (not ((n[curNode^]^.thisUser.SecLevel = 255) and (privates^^.the_options.unlim_sysop_play))) then
						begin
							bCR(procs[0]);
							prt(concat('Hey, you leech!  Only ', cstr(privates^^.the_options.plays_allowed), ' games per day!'), HermSetup);
							bCR(procs[0]);
							privates^^.successful := false
						end
					else
						privates^^.successful := true
				end
	end;

	procedure format_drive (var HermSetup: UserXIPtr);
	begin
		with HermSetup^.privates^^.the_leech[HermSetup^.curNode^] do
			begin
				Level := 1;
				CurMegs := 10;
				MegsMax := 10;
				Software := 'Dumb Terminal';
				SoftwarePower := 1;
				Backup := 'None';
				BackupPower := 1;
				Money := rnd(200, HermSetup);
				DrivesToCrash := 10;
				Bank := 0;
				DrivesToCrash := Level * 10
			end
	end;

	procedure alert_LLL_battle (var HermSetup: UserXIPtr);	{alert leech to impending Final Battle against the LLL computer}
	begin
		with HermSetup^ do
			begin
				OutChr(chr(12), procs[12]);
				prt(concat('Congratulations, ', privates^^.the_leech[curNode^].Name, '!'), HermSetup);
				print_text_rsrc(final_battle_txt, HermSetup);
				privates^^.the_leech[curNode^].CurMegs := 310;
				privates^^.the_enemy[curNode^].CurMegs := 360;
				privates^^.the_enemy[curNode^].SoftwarePower := privates^^.the_leech[curNode^].SoftwarePower;
				privates^^.the_enemy[curNode^].BackupPower := privates^^.the_leech[curNode^].BackupPower
			end
	end;

	procedure convert_software (the_software: integer; var power: integer; var cost: longint; var name: str255);
  {convert a numbered software to power, cost, and name}
	begin
		case the_software of
			1: 
				begin
					power := 1;
					cost := 0;
					name := 'Dumb Terminal'
				end;
			2: 
				begin
					power := 2;
					cost := 100;
					name := 'Turbo-Term'
				end;
			3: 
				begin
					power := 3;
					cost := 200;
					name := 'SmartCom'
				end;
			4: 
				begin
					power := 4;
					cost := 400;
					name := 'QModem SST'
				end;
			5: 
				begin
					power := 5;
					cost := 800;
					name := 'ProComm'
				end;
			6: 
				begin
					power := 7;
					cost := 2000;
					name := 'Hacker''s Delight'
				end;
			7: 
				begin
					power := 8;
					cost := 4000;
					name := 'Trojan Generator'
				end;
			8: 
				begin
					power := 9;
					cost := 8000;
					name := 'ANSI Porno Picture'
				end;
			9: 
				begin
					power := 11;
					cost := 20000;
					name := 'Dumb Comm'
				end;
			10: 
				begin
					power := 12;
					cost := 40000;
					name := 'Keyboard Locker'
				end;
			11: 
				begin
					power := 13;
					cost := 80000;
					name := 'ASCII Express'
				end;
			12: 
				begin
					power := 15;
					cost := 2000000;
					name := 'ProTerm'
				end;
			13: 
				begin
					power := 16;
					cost := 4000000;
					name := 'Hermes Terminal Prog.'
				end;
			14: 
				begin
					power := 17;
					cost := 8000000;
					name := 'Super Deleter'
				end;
			15: 
				begin
					power := 20;
					cost := 20000000;
					name := 'WipeOut Comm'
				end;
			otherwise
		end
	end;

	procedure convert_backup (the_backup: integer; var power: integer; var cost: longint; var name: str255);
  {convert a numbered backup to power, cost, and name}
	begin
		case the_backup of
			1: 
				begin
					power := 1;
					cost := 0;
					name := 'None'
				end;
			2: 
				begin
					power := 2;
					cost := 100;
					name := 'Punch Cards'
				end;
			3: 
				begin
					power := 3;
					cost := 200;
					name := 'Paper Strips'
				end;
			4: 
				begin
					power := 4;
					cost := 400;
					name := 'Floppy Disk'
				end;
			5: 
				begin
					power := 5;
					cost := 800;
					name := 'High Density Floppy'
				end;
			6: 
				begin
					power := 7;
					cost := 2000;
					name := 'Tape Backup'
				end;
			7: 
				begin
					power := 8;
					cost := 4000;
					name := 'Long Tape Backup'
				end;
			8: 
				begin
					power := 9;
					cost := 8000;
					name := 'Elephant'
				end;
			9: 
				begin
					power := 11;
					cost := 20000;
					name := 'Duplicate Computer'
				end;
			10: 
				begin
					power := 12;
					cost := 40000;
					name := 'Dup. Computer w/Lock'
				end;
			11: 
				begin
					power := 13;
					cost := 80000;
					name := 'Federal Protection'
				end;
			12: 
				begin
					power := 15;
					cost := 2000000;
					name := 'Worldwide Protection'
				end;
			13: 
				begin
					power := 16;
					cost := 4000000;
					name := 'Teenager to Busy Line'
				end;
			14: 
				begin
					power := 17;
					cost := 8000000;
					name := 'Read-Only Attribute'
				end;
			15: 
				begin
					power := 20;
					cost := 20000000;
					name := 'All Backups'
				end;
			otherwise
		end
	end;

	procedure print (var ref: integer; var HermSetup: UserXIPtr; n: string);	{print string n to text file #ref}
		var
			t: string;
			err: OSErr;
			temp_l: longint;
	begin
		t := concat(n, chr(13), chr(10));
		temp_l := length(t);
		err := FSWrite(ref, temp_l, @t[1]);
		parseerr(err, HermSetup)
	end;

	procedure print_current_money (var HermSetup: UserXIPtr);	{used to save code space}
	begin
		prt(concat('You have $', cstrl(HermSetup^.privates^^.the_leech[HermSetup^.curNode^].Money), '.'), HermSetup)
	end;

	procedure write_user_status (var HermSetup: UserXIPtr);	{display the leech's statistics}
	begin
		with HermSetup^ do
			with privates^^.the_leech[curNode^] do
				begin
					prt('User Stats', HermSetup);
					prt('----------', HermSetup);
					prt(concat('Leech: ', Name), HermSetup);
					prt(concat('Leech #', cstr(RealNumber)), HermSetup);
					prt(concat('Level: ', cstr(Level)), HermSetup);
					prt(concat('Software: ', Software), HermSetup);
					prt(concat('Backup: ', Backup), HermSetup);
					prt(concat('Drives left to crash: ', cstr(DrivesToCrash)), HermSetup);
					prt(concat('Money: $', cstrl(Money)), HermSetup);
					prt(concat('Money in LLL account: $', cstrl(Bank)), HermSetup);
					prt(concat('Games left today: ', cstr(privates^^.the_options.plays_allowed - TimesOnToday)), HermSetup);
					prt(concat('Fast backups left: ', cstr(FastBacks)), HermSetup);
					prt(concat('Trojan Horses left: ', cstr(Trojans)), HermSetup);
					prt(concat('Calls to BBS''s left: ', cstr(CallsToday)), HermSetup);
					prt(concat('Calls to leeches left: ', cstr(HacksToday)), HermSetup);
					prt(concat('Megs left: ', cstr(CurMegs), '(', cstr(MegsMax), ')'), HermSetup);
					bCR(procs[0])
				end
	end;

	procedure write_bulletin_file (var HermSetup: UserXIPtr);	{create a new bulletin file}
		var
			err: OSErr;
			ref_num: integer;
	begin
		with HermSetup^ do
			if n[curNode^]^.curPrompt[1] = 'Y' then
				begin
					err := FSDelete(concat(filesPath^, data_path, bulletin_fName), 0);
					parseerr(err, HermSetup);
					err := Create(concat(filesPath^, data_path, bulletin_fName), 0, creator_type, text_type);
					parseerr(err, HermSetup);
					err := FSOpen(concat(filesPath^, data_path, bulletin_fName), 0, ref_num);
					parseerr(err, HermSetup);
					print(ref_num, HermSetup, '');
					print(ref_num, HermSetup, concat('Bulletin posted on ', privates^^.the_leech[curNode^].DateCalled, ' by ', privates^^.the_leech[curNode^].Name, '.'));
					print(ref_num, HermSetup, divide_line);
					print(ref_num, HermSetup, privates^^.s1);
					print(ref_num, HermSetup, privates^^.s2);
					print(ref_num, HermSetup, privates^^.s3);
					print(ref_num, HermSetup, divide_line);
					err := FSClose(ref_num);
					parseerr(err, HermSetup)
				end
	end;

	procedure hack_cards (var HermSetup: UserXIPtr);

		function total_bank (var HermSetup: UserXIPtr): longint;	{total amount of money in LLL bank}
			var
				temp_leech: leech_rec;
				i, ref_num: integer;
				temp, the_longint, the_longint2: longint;
				err: OSErr;
		begin
			temp := 0;
			err := FSOpen(concat(HermSetup^.filesPath^, data_path, leech_data_fName), 0, ref_num);
			parseerr(err, HermSetup);
			err := GetEOF(ref_num, the_longint);
			parseerr(err, HermSetup);
			the_longint2 := SizeOf(leech_rec);
			for i := 1 to (the_longint div SizeOf(leech_rec)) do
				begin
					err := FSRead(ref_num, the_longint2, @temp_leech);
					parseerr(err, HermSetup);
					temp := temp + temp_leech.Bank
				end;
			err := FSClose(ref_num);
			parseerr(err, HermSetup);
			total_bank := temp
		end;

		procedure erase_bank (var HermSetup: UserXIPtr);	{erase the LLL bank}
			var
				err: OSErr;
				temp_leech: leech_rec;
				i, ref_num: integer;
				the_longint, the_longint2: longint;
		begin
			err := FSOpen(concat(HermSetup^.filesPath^, data_path, leech_data_fName), 0, ref_num);
			parseerr(err, HermSetup);
			err := GetEOF(ref_num, the_longint);
			parseerr(err, HermSetup);
			the_longint2 := SizeOf(leech_rec);
			for i := 1 to (the_longint div SizeOf(leech_rec)) do
				begin
					err := FSRead(ref_num, the_longint2, @temp_leech);
					parseerr(err, HermSetup);
					temp_leech.Bank := 0;
					err := SetFPos(ref_num, fsFromStart, longint(i - 1) * SizeOf(leech_rec));
					parseerr(err, HermSetup);
					err := FSWrite(ref_num, the_longint2, @temp_leech);
					parseerr(err, HermSetup)
				end;
			err := FSClose(ref_num);
			parseerr(err, HermSetup)
		end;

	begin
		with HermSetup^ do
			with privates^^ do
				with privates^^.the_leech[curNode^] do
					begin
						HacksToday := HacksToday - 1;
						t1 := trunc((DrivesToCrash * 10) / SoftwarePower);
						if t1 < 2 then
							t1 := 2;
						t2 := rnd(t1, HermSetup);
						if t2 = 1 then
							begin
								t1 := total_bank(HermSetup);
								erase_bank(HermSetup);
								Bank := Bank + t1;
								prt('You successfully hacked out cards!', HermSetup);
								prt(concat('You transferred $', cstr(t1), ' to your account.'), HermSetup)
							end
						else
							begin
								prt('You were caught.  The LLL now requires you to crash more hard disks to', HermSetup);
								prt('go up another level.', HermSetup);
								DrivesToCrash := DrivesToCrash + (Level * 5)
							end
					end
	end;

	procedure list_leeches (var HermSetup: UserXIPtr);	{list the leeches in the leech data file}
		var
			i, ref_num: integer;
			err: OSErr;
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					bCR(procs[0]);
					prt('List of registered leeches:', HermSetup);
					bCR(procs[0]);
					prt('Name                            #     Level', HermSetup);
					prt('-------------------------------------------', HermSetup);
					err := FSOpen(concat(filesPath^, data_path, leech_data_fName), 0, ref_num);
					parseerr(err, HermSetup);
					t1 := 31;
					err := GetEOF(ref_num, l1);
					parseerr(err, HermSetup);
					repeat
						for i := 1 to l1 div SizeOf(leech_rec) do
							begin
								read_usernum(i, HermSetup, temp_leech[curNode^], ref_num);
								if temp_leech[curNode^].Level = t1 then
									prt(concat(space_it(temp_leech[curNode^].Name, 32), space_it(cstr(temp_leech[curNode^].RealNumber), 8), space_it(cstr(temp_leech[curNode^].Level), 4)), HermSetup)
							end;
						t1 := t1 - 1
					until (t1 <= 0);
					err := FSClose(ref_num);
					parseerr(err, HermSetup)
				end
	end;

	procedure gamble (var HermSetup: UserXIPtr);
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					t1 := s2n(n[curNode^]^.curPrompt);
					if (t1 < 1) then
						prt('Invalid bet.', HermSetup)
					else
						begin
							prt('Flipping...', HermSetup);
							t2 := rnd(1, HermSetup);
							if t2 = 0 then
								begin
									prt(concat('You won $', cstr(t1), '!!'), HermSetup);
									the_leech[curNode^].Money := the_leech[curNode^].Money + t1
								end
							else
								begin
									the_leech[curNode^].Money := the_leech[curNode^].Money - t1;
									prt('Whoops!  You lost!', HermSetup)
								end
						end
				end
	end;

	procedure swap (var x, y: integer);	{swap integers x with y}
		var
			temp: integer;
	begin
		temp := x;
		x := y;
		y := temp
	end;

	procedure swap_string (var x, y: str255);	{swap strings x with y}
		var
			temp: str255;
	begin
		temp := x;
		x := y;
		y := temp
	end;

	procedure trash_disk (var HermSetup: UserXIPtr);	{Battle: standard form of attack versus a computer}
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					t2 := trunc((the_leech[curNode^].SoftwarePower * rnd(10, HermSetup)) / the_enemy[curNode^].BackupPower);
					t3 := trunc((the_enemy[curNode^].SoftwarePower * rnd(10, HermSetup)) / the_leech[curNode^].BackupPower);
					if t2 >= 1 then
						prt(concat('You trash ', cstr(t2), ' megs!'), HermSetup)
					else
						prt('You got line noise!', HermSetup);
					the_enemy[curNode^].CurMegs := the_enemy[curNode^].CurMegs - t2;
					if the_enemy[curNode^].CurMegs > 0 then
						begin
							if t3 >= 1 then
								prt(concat('They purge ', cstr(t3), ' megs!'), HermSetup)
							else
								prt('They deleted an EMPTY directory!', HermSetup);
							the_leech[curNode^].CurMegs := the_leech[curNode^].CurMegs - t3
						end;
					if (rnd(20, HermSetup) = 6) and (the_enemy[curNode^].CurMegs > 0) then
						begin
							t2 := rnd(5, HermSetup) * the_leech[curNode^].BackupPower;
							if t2 > 0 then
								begin
									prt('Your hard disk has a File Allocation Table error!', HermSetup);
									prt(concat('Say goodbye to ', cstr(t2), ' megs!'), HermSetup);
									bCR(procs[0]);
									the_leech[curNode^].CurMegs := the_leech[curNode^].CurMegs - t2
								end
						end
				end
	end;

	procedure drop_carrier (var HermSetup: UserXIPtr);	{Battle: Abort battle}
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					bCR(procs[0]);
					prt('(ha!)', HermSetup);
					prt('NO CARRIER', HermSetup);
					bCR(procs[0]);
					bCR(procs[0]);
					is_hacking := false;
					if the_leech[curNode^].Level = 31 then
						begin
							prt('The LLL is pissed.  Go back and repeat level 30!', HermSetup);
							the_leech[curNode^].Level := 30;
							the_leech[curNode^].DrivesToCrash := 300;
							write_user(the_leech[curNode^], HermSetup)
						end
				end
	end;

	procedure fast_backup (var HermSetup: UserXIPtr);	{Battle: Fast backup to recover some megs}
	begin
		with HermSetup^ do
			with privates^^.the_leech[curNode^] do
				if FastBacks < 1 then
					prt('The files were completely purged!', HermSetup)
				else
					begin
						FastBacks := FastBacks - 1;
						privates^^.t2 := rnd(5, HermSetup) * BackupPower;
						prt('You run a backup program...', HermSetup);
						if (CurMegs + privates^^.t2) >= MegsMax then
							begin
								prt('You recover all of your megs back!', HermSetup);
								CurMegs := MegsMax
							end
						else
							begin
								prt(concat('You recover ', cstr(privates^^.t2), ' megs back!'), HermSetup);
								CurMegs := CurMegs + privates^^.t2;
								if CurMegs > MegsMax then
									CurMegs := MegsMax
							end
					end
	end;

	procedure upload_trojan (var HermSetup: UserXIPtr);	{Battle: Upload trojan to try to damage enemy}
	begin
		with HermSetup^ do
			with privates^^.the_leech[curNode^] do
				if Trojans < 1 then
					prt('Oops!  You accidentally erased it!', HermSetup)
				else
					begin
						Trojans := Trojans - 1;
						privates^^.t2 := rnd(5, HermSetup) * SoftwarePower;
						prt('You upload a trojan horse...', HermSetup);
						prt(concat('The idiot runs it! ', cstr(privates^^.t2), ' megs are trashed!'), HermSetup);
						privates^^.the_enemy[curNode^].CurMegs := privates^^.the_enemy[curNode^].CurMegs - privates^^.t2
					end
	end;

	procedure update (var HermSetup: UserXIPtr);	{Battle: Update 1}
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					temp_leech[curNode^].Money := temp_leech[curNode^].Money + the_leech[curNode^].Money;
					the_leech[curNode^].Money := 0;
					temp_leech[curNode^].DrivesToCrash := temp_leech[curNode^].DrivesToCrash - (the_leech[curNode^].Level * 3);
					if temp_leech[curNode^].DrivesToCrash <= 0 then
						begin
							temp_leech[curNode^].Level := temp_leech[curNode^].Level + 1;
							temp_leech[curNode^].DrivesToCrash := temp_leech[curNode^].Level * 10
						end;
					if temp_leech[curNode^].SoftwarePower < the_leech[curNode^].SoftwarePower then
						begin
							bCR(procs[0]);
							prt('He downloaded your software!', HermSetup);
							swap(temp_leech[curNode^].SoftwarePower, the_leech[curNode^].SoftwarePower);
							swap_string(temp_leech[curNode^].Software, the_leech[curNode^].Software)
						end;
					if temp_leech[curNode^].BackupPower < the_leech[curNode^].BackupPower then
						begin
							bCR(procs[0]);
							prt('He stole your backup!', HermSetup);
							swap(temp_leech[curNode^].BackupPower, the_leech[curNode^].BackupPower);
							swap_string(temp_leech[curNode^].Backup, the_leech[curNode^].Backup)
						end;
					write_user(the_leech[curNode^], HermSetup);
					write_user(temp_leech[curNode^], HermSetup);
					is_hacking := false
				end
	end;

	procedure update2 (var HermSetup: UserXIPtr);	{Battle: Update 2}
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					temp_leech[curNode^].Money := 0;
					bCR(procs[0]);
					if the_leech[curNode^].Level >= temp_leech[curNode^].Level then
						begin
							the_leech[curNode^].DrivesToCrash := the_leech[curNode^].DrivesToCrash - (temp_leech[curNode^].Level * 3);
							if the_leech[curNode^].DrivesToCrash > 0 then
								prt('The LLL is proud and will give you credit for this.', HermSetup)
							else
								prt('The LLL couldn''t be more proud!  They promote you a level!', HermSetup)
						end
					else
						begin
							swap(the_leech[curNode^].Level, temp_leech[curNode^].Level);
							swap(the_leech[curNode^].DrivesToCrash, temp_leech[curNode^].DrivesToCrash);
							prt(concat('The LLL promotes you to level ', cstr(the_leech[curNode^].Level), '!'), HermSetup);
							bCR(procs[0]);
							prt(concat('They demote him to level ', cstr(temp_leech[curNode^].Level), '!'), HermSetup);
							bCR(procs[0])
						end;
					if temp_leech[curNode^].SoftwarePower > the_leech[curNode^].SoftwarePower then
						begin
							prt('You downloaded his software!', HermSetup);
							swap(temp_leech[curNode^].SoftwarePower, the_leech[curNode^].SoftwarePower);
							swap_string(temp_leech[curNode^].Software, the_leech[curNode^].Software)
						end;
					if temp_leech[curNode^].BackupPower > the_leech[curNode^].BackupPower then
						begin
							prt('You stole his backup!', HermSetup);
							swap(temp_leech[curNode^].BackupPower, the_leech[curNode^].BackupPower);
							swap_string(temp_leech[curNode^].Backup, the_leech[curNode^].Backup)
						end;
					write_user(the_leech[curNode^], HermSetup);
					write_user(temp_leech[curNode^], HermSetup);
					is_hacking := false
				end
	end;

	procedure advance_level (var HermSetup: UserXIPtr);	{Battle: Advance a leech a level}
	begin
		with HermSetup^ do
			with privates^^.the_leech[curNode^] do
				begin
					Level := Level + 1;
					bCR(procs[0]);
					prt(concat('The LLL is pleased with this performance.  You are now level ', cstr(Level), '.'), HermSetup);
					bCR(procs[0]);
					CurMegs := CurMegs + 10;
					MegsMax := MegsMax + 10;
					DrivesToCrash := 10 * Level;
					Trojans := Trojans + 1;
					FastBacks := FastBacks + 1
				end
	end;

	procedure they_dead (var HermSetup: UserXIPtr);	{Battle: Alert leech to enemy's death}
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					prt('Their computer is FRIED!', HermSetup);
					prt(concat('And look, here''s his credit card # with $', cstrl(the_enemy[curNode^].Money), ' in his account!'), HermSetup);
					the_leech[curNode^].Money := the_leech[curNode^].Money + the_enemy[curNode^].Money;
					if not is_hacking then
						the_leech[curNode^].DrivesToCrash := the_leech[curNode^].DrivesToCrash - t1
					else
						update2(HermSetup);
					bCR(procs[0]);
					print_current_money(HermSetup);
					if the_leech[curNode^].DrivesToCrash <= 0 then
						advance_level(HermSetup);
					write_user(the_leech[curNode^], HermSetup)
				end
	end;

	procedure you_dead (var HermSetup: UserXIPtr);	{Battle: Alert leech to his death}
	begin
		with HermSetup^ do
			with privates^^ do
				begin
					bCR(procs[0]);
					prt('There goes your hard disk!', HermSetup);
					if not is_hacking then
						the_leech[curNode^].Money := round(the_leech[curNode^].Money / 2)
					else
						update(HermSetup);
					if the_leech[curNode^].Level = 31 then
						begin
							the_leech[curNode^].Level := 30;
							the_leech[curNode^].DrivesToCrash := 300
						end
				end
	end;

	procedure add_to_halloffame (var HermSetup: UserXIPtr);	{Add leech to Hall of Fame when LLL computer is beaten}
		var
			err: OSErr;
			the_longint: longint;
			ref_num: integer;
	begin
		err := FSOpen(concat(HermSetup^.filesPath^, data_path, hall_fName), 0, ref_num);
		parseerr(err, HermSetup);
		err := GetEOF(ref_num, the_longint);
		parseerr(err, HermSetup);
		err := SetFPos(ref_num, fsFromStart, the_longint);
		parseerr(err, HermSetup);
		print(ref_num, HermSetup, HermSetup^.privates^^.the_leech[HermSetup^.curNode^].Name);
		err := FSClose(ref_num);
		parseerr(err, HermSetup)
	end;

	procedure reset_game (var HermSetup: UserXIPtr);	{reset whole game once a new leech has beaten the LLL computer}
		var
			err: OSErr;
			ref_num: integer;
	begin
		with HermSetup^ do
			begin
				err := FSDelete(concat(filesPath^, data_path, leech_data_fName), 0);
				parseerr(err, HermSetup);
				err := FSDelete(concat(filesPath^, data_path, bulletin_fName), 0);
				parseerr(err, HermSetup);
				err := Create(concat(filesPath^, data_path, bulletin_fName), 0, creator_type, text_type);
				parseerr(err, HermSetup);
				err := FSOpen(concat(filesPath^, data_path, bulletin_fName), 0, ref_num);
				parseerr(err, HermSetup);
				print(ref_num, HermSetup, '');
				print(ref_num, HermSetup, divide_line);
				print(ref_num, HermSetup, concat('Congratulations to ', privates^^.the_leech[curNode^].Name, ', the winner of the'));
				print(ref_num, HermSetup, 'previous game of Leech!  A new game of Leech has been started.  Good luck!');
				print(ref_num, HermSetup, divide_line);
				err := FSClose(ref_num);
				parseerr(err, HermSetup)
			end
	end;

	procedure get_the_bbs (which: integer; var name, number: str255; var HermSetup: UserXIPtr);
		var
			the_text: CharsHandle;
			i, the_loc: integer;
			finished: boolean;
	begin
		the_text := CharsHandle(GetResource(text_type, board_data_txt));
		parseerr(ResError, HermSetup);
		the_loc := -1;
		finished := false;
		for i := 1 to which do
			begin
				extract_line(the_loc, the_text, name, finished);
				extract_line(the_loc, the_text, number, finished)
			end;
		ReleaseResource(handle(the_text));
		parseerr(ResError, HermSetup)
	end;

	procedure dly (how_long: integer);
		var
			the_longint: longint;
	begin
		delay(how_long, the_longint)
	end;

	procedure main (HermSetup: UserXIPtr);
		var
			i: integer;
			menu_cmd, bbs_name, bbs_number: str255;
			the_user: UserRec;
	begin
		with HermSetup^ do
			begin
				if message <> closeExternal then
					begin
						if privates = nil then  {never been called before}
							begin
								privates := myPrivsHand(NewHandle(SizeOf(myPrivs)));
								MoveHHi(handle(privates));
								HNoPurge(handle(privates));
								for i := 1 to 10 do
									begin
										privates^^.activeOn[i] := false;
										privates^^.stage[i] := 0
									end;
								privates^^.is_hacking := false;
								read_option_rec(privates^^.the_options)
							end;
						if not privates^^.activeOn[curNode^] then
							begin
								privates^^.activeOn[curNode^] := true;
								privates^^.stage[curNode^] := 1
							end
					end;
				case message of
					closeExternal: 
						if privates <> nil then
							begin
								HPurge(handle(privates));
								DisposHandle(handle(privates));
								privates := nil
							end;
					closeNode: 
						begin
							privates^^.activeOn[curNode^] := false;
							privates^^.stage[curNode^] := 0
						end;
					activeExt: 
						case privates^^.stage[curNode^] of
							1: 	{display opening screen}
								begin
									OutChr(chr(12), procs[12]);
									print_text_rsrc(opening_txt, HermSetup);
									privates^^.pReturnStage := 2;
									privates^^.stage[curNode^] := pause_stage
								end;
							2: 	{display hall of fame}
								begin
									OutChr(chr(12), procs[12]);
									write_halloffame(HermSetup);
									privates^^.stage[curNode^] := 3
								end;
							3: 	{read the user from the user file, or initialize user if new, and check to make sure he can play}
								begin
									read_user(HermSetup);
									bCR(procs[0]);
									check_user(HermSetup);
									if privates^^.successful then
										privates^^.stage[curNode^] := 5
									else
										privates^^.stage[curNode^] := exit_stage
								end;
							5: 	{display bulletin}
								begin
									bCR(procs[0]);
									if ReadTextFile(concat(data_path, bulletin_fName), 0, true, procs[10]) then
										n[curNode^]^.BoardAction := ListText;
									privates^^.stage[curNode^] := 6
								end;
							6: 
								begin
									bCR(procs[0]);
									prt('Booting up...', HermSetup);
									privates^^.stage[curNode^] := main_menu_stage
								end;
							main_menu_stage: {main menu prompt}
								if privates^^.the_leech[curNode^].Level >= 31 then
									begin
										alert_LLL_battle(HermSetup);
										privates^^.pReturnStage := fight_stage;
										privates^^.stage[curNode^] := pause_stage
									end
								else
									begin
										bCR(procs[0]);
										prt('"?" to see menu.', HermSetup);
										bCR(procs[0]);
										LettersPrompt('] ', '', 7, false, false, true, char(0), procs[4]);
										privates^^.stage[curNode^] := 8
									end;
							8: 	{parse main menu}
								begin
									menu_cmd := n[curNode^]^.curPrompt;
									bCR(procs[0]);
									bCR(procs[0]);
									if (menu_cmd = '?') or (menu_cmd = 'DIR') or (menu_cmd = 'CATALOG') or (menu_cmd = 'CAT') or (menu_cmd = 'LS') then
										privates^^.stage[curNode^] := 9
									else if menu_cmd = 'HELP' then
										privates^^.stage[curNode^] := 12
									else if menu_cmd = 'QUIT' then
										privates^^.stage[curNode^] := 10
									else if menu_cmd = 'UE' then
										privates^^.stage[curNode^] := 13
									else if menu_cmd = 'STATUS' then
										privates^^.stage[curNode^] := 15
									else if menu_cmd = '+' then
										privates^^.stage[curNode^] := 16
									else if menu_cmd = '-' then
										privates^^.stage[curNode^] := 17
									else if menu_cmd = 'CREDITS' then
										privates^^.stage[curNode^] := 18
									else if menu_cmd = 'LLL' then
										begin
											prt('Dialing LLL...', HermSetup);
											bCR(procs[0]);
											prt('CONNECT', HermSetup);
											bCR(procs[0]);
											privates^^.stage[curNode^] := lll_menu_stage
										end
									else if menu_cmd = 'CALL' then
										begin
											OutChr(chr(12), procs[12]);
											privates^^.stage[curNode^] := 50
										end
									else if menu_cmd = 'FORMAT' then
										privates^^.stage[curNode^] := 60
									else if menu_cmd = 'HACK' then
										privates^^.stage[curNode^] := 62
									else
										begin
											OutChr(chr(7), procs[12]);
											prt('? SYNTAX ERROR ?', HermSetup);
											privates^^.stage[curNode^] := main_menu_stage
										end
								end;
							9: 	{display main menu}
								begin
									print_text_rsrc(main_menu_txt, HermSetup);
									privates^^.stage[curNode^] := main_menu_stage
								end;
							10: 	{check whether to quit}
								begin
									YesNoQuestion('Quit Leech?  Are you sure? ', false, procs[6]);
									privates^^.stage[curNode^] := 11
								end;
							11: 	{""}
								if n[curNode^]^.curPrompt[1] = 'Y' then
									privates^^.stage[curNode^] := exit_stage
								else
									privates^^.stage[curNode^] := main_menu_stage;
							12: 	{display help file}
								begin
									if ReadTextFile(concat(data_path, help_fName), 0, true, procs[10]) then
										n[curNode^]^.BoardAction := ListText;
									privates^^.stage[curNode^] := main_menu_stage
								end;
							13: 	{unerase megs; recover damaged megs}
								with privates^^ do
									begin
										t1 := the_leech[curNode^].MegsMax - the_leech[curNode^].CurMegs;
										if t1 = 0 then
											begin
												prt('Your hard disk has no damage!', HermSetup);
												bCR(procs[0]);
												stage[curNode^] := main_menu_stage
											end
										else
											begin
												prt(concat('You have ', cstr(t1), ' damaged megs.'), HermSetup);
												t2 := the_leech[curNode^].Level * 10;	{cost to repair each meg}
												prt(concat('It will cost you $', cstr(t2), ' to repair each meg.'), HermSetup);
												print_current_money(HermSetup);
												bCR(procs[0]);
												NumbersPrompt('Recover how many megs - ', '', 32767, 0, procs[5]);
												stage[curNode^] := 14
											end;
									end;
							14: 	{""}
								with privates^^ do
									begin
										t3 := s2n(n[curNode^]^.curPrompt);	{megs to recover}
										if t3 > t1 then
											prt('You don''t have that many damaged megs.', HermSetup)
										else if t3 * t2 > the_leech[curNode^].Money then
											prt(not_enough_money, HermSetup)
										else
											begin
												the_leech[curNode^].Money := the_leech[curNode^].Money - (t3 * t2);
												the_leech[curNode^].CurMegs := the_leech[curNode^].CurMegs + t3;
												t1 := t1 - t3;
												prt(concat('You now have $', cstrl(the_leech[curNode^].Money), ' and ', cstr(t1), ' damaged megs.'), HermSetup)
											end;
										bCR(procs[0]);
										stage[curNode^] := main_menu_stage
									end;
							15: 	{display user status}
								begin
									write_user_status(HermSetup);
									privates^^.stage[curNode^] := main_menu_stage
								end;
							16: 	{add money to bank}
								with privates^^.the_leech[curNode^] do
									begin
										Bank := Bank + Money;
										Money := 0;
										prt(concat('Your LLL account now has $', cstrl(Bank), '.'), HermSetup);
										bCR(procs[0]);
										privates^^.stage[curNode^] := main_menu_stage
									end;
							17: 	{remove money from bank}
								with privates^^.the_leech[curNode^] do
									begin
										Money := Money + Bank;
										Bank := 0;
										prt(concat('You are now carrying $', cstrl(Money), '.'), HermSetup);
										bCR(procs[0]);
										privates^^.stage[curNode^] := main_menu_stage
									end;
							18: 	{display credits}
								begin
									prt(concat('Leech! Online Game - Version ', vers_num), HermSetup);
									print_text_rsrc(credits_txt, HermSetup);
									privates^^.stage[curNode^] := main_menu_stage
								end;
							lll_menu_stage: 	{LLL menu prompt}
								begin
									bCR(procs[0]);
									bCR(procs[0]);
									prt('(Lame Local Leeches BBS - "?" for menu) - ', HermSetup);
									LettersPrompt('', 'BCEFGHLSTVQ?', 1, true, false, true, char(0), procs[4]);
									privates^^.stage[curNode^] := 20
								end;
							20: 	{parse LLL menu}
								if length(n[curNode^]^.curPrompt) > 0 then
									case n[curNode^]^.curPrompt[1] of
										'B': 
											privates^^.stage[curNode^] := 21;
										'C': 
											privates^^.stage[curNode^] := 26;
										'E': 
											privates^^.stage[curNode^] := 29;
										'F': 
											privates^^.stage[curNode^] := 65;
										'G': 
											privates^^.stage[curNode^] := 33;
										'H': 
											privates^^.stage[curNode^] := 35;
										'L': 
											privates^^.stage[curNode^] := 40;
										'S': 
											privates^^.stage[curNode^] := 43;
										'T': 
											privates^^.stage[curNode^] := 68;
										'V': 
											begin
												write_halloffame(HermSetup);
												privates^^.stage[curNode^] := lll_menu_stage
											end;
										'?': 
											privates^^.stage[curNode^] := 28;
										'Q': 
											begin
												bCR(procs[0]);
												prt('NO CARRIER', HermSetup);
												privates^^.stage[curNode^] := main_menu_stage
											end;
										otherwise
											privates^^.stage[curNode^] := lll_menu_stage
									end
								else
									privates^^.stage[curNode^] := lll_menu_stage;
							21: 	{create new bulletin}
								begin
									prt('Enter three lines for a bulletin that each Leech player will see:', HermSetup);
									bCR(procs[0]);
									LettersPrompt('1: ', '', 76, false, false, false, char(0), procs[4]);
									ANSIPrompter(76, procs[9]);
									privates^^.stage[curNode^] := 22
								end;
							22: 	{""}
								begin
									privates^^.s1 := n[curNode^]^.curPrompt;
									LettersPrompt('2: ', '', 76, false, false, false, char(0), procs[4]);
									ANSIPrompter(76, procs[9]);
									privates^^.stage[curNode^] := 23
								end;
							23: 	{""}
								begin
									privates^^.s2 := n[curNode^]^.curPrompt;
									LettersPrompt('3: ', '', 76, false, false, false, char(0), procs[4]);
									ANSIPrompter(76, procs[9]);
									privates^^.stage[curNode^] := 24
								end;
							24: 	{""}
								begin
									privates^^.s3 := n[curNode^]^.curPrompt;
									YesNoQuestion('Is this what you want it to say? ', true, procs[6]);
									privates^^.stage[curNode^] := 25
								end;
							25: 	{""}
								begin
									write_bulletin_file(HermSetup);
									privates^^.stage[curNode^] := lll_menu_stage
								end;
							26: 	{hack a credit card #}
								begin
									prt('Failing to hack a credit card will get the LLL *MAD*.', HermSetup);
									bCR(procs[0]);
									YesNoQuestion('Still hack? ', true, procs[6]);
									privates^^.stage[curNode^] := 27
								end;
							27: 	{""}
								begin
									bCR(procs[0]);
									if n[curNode^]^.curPrompt[1] = 'Y' then
										if privates^^.the_leech[curNode^].HacksToday <= 0 then
											prt('Sorry, you can only try to hack a card three times per game.', HermSetup)
										else
											hack_cards(HermSetup);
									privates^^.stage[curNode^] := lll_menu_stage
								end;
							28: 	{display LLL menu}
								begin
									print_text_rsrc(LLL_menu_txt, HermSetup);
									privates^^.stage[curNode^] := lll_menu_stage
								end;
							29: 	{examine a leech}
								begin
									LettersPrompt('Enter the name or number of the leech to examine: ', '', 31, false, false, true, char(0), procs[4]);
									ANSIPrompter(31, procs[9]);
									privates^^.stage[curNode^] := 30
								end;
							30: 	{""}
								if FindUser(n[curNode^]^.curPrompt, the_user, procs[15]) then
									begin
										find_user(the_user.UserNum, privates^^.the_leech[curNode^].RealNumber, HermSetup, privates^^.temp_leech[curNode^]);
										if privates^^.successful then
											privates^^.stage[curNode^] := 31
										else
											begin
												prt(inval_leech, HermSetup);
												privates^^.stage[curNode^] := lll_menu_stage
											end
									end
								else
									begin
										prt(inval_leech, HermSetup);
										privates^^.stage[curNode^] := lll_menu_stage
									end;
							31: 	{""}
								begin
									privates^^.t2 := privates^^.the_leech[curNode^].Level * 10;
									bCR(procs[0]);
									YesNoQuestion(concat('Are you aware of the fact that it will cost you $', cstr(privates^^.t2), '? '), true, procs[6]);
									privates^^.stage[curNode^] := 32
								end;
							32: 	{""}
								begin
									if n[curNode^]^.curPrompt[1] = 'Y' then
										begin
											bCR(procs[0]);
											if privates^^.the_leech[curNode^].Money - privates^^.t2 < 0 then
												prt(not_enough_money, HermSetup)
											else
												begin
													privates^^.the_leech[curNode^].Money := privates^^.the_leech[curNode^].Money - privates^^.t2;
													with privates^^.temp_leech[curNode^] do
														begin
															prt(concat('Leech: ', Name), HermSetup);
															prt(concat('Leech #', cstr(RealNumber)), HermSetup);
															prt(concat('Level: ', cstr(Level)), HermSetup);
															prt(concat('Software: ', Software), HermSetup);
															prt(concat('Backup: ', Backup), HermSetup);
															prt(concat('Money: $', cstrl(Money)), HermSetup);
															prt(concat('Megs left: ', cstr(MegsMax), '(', cstr(MegsMax), ')'), HermSetup)
														end
												end
										end;
									privates^^.stage[curNode^] := lll_menu_stage
								end;
							33: 	{gamble}
								begin
									prt('A flip of the coin, eh?', HermSetup);
									bCR(procs[0]);
									print_current_money(HermSetup);
									bCR(procs[0]);
									NumbersPrompt('How much do you want to gamble? ', '', privates^^.the_leech[curNode^].Money, 0, procs[5]);
									privates^^.stage[curNode^] := 34
								end;
							34: 	{""}
								begin
									gamble(HermSetup);
									privates^^.stage[curNode^] := lll_menu_stage
								end;
							35: 	{buy/sell backup menu}
								begin
									bCR(procs[0]);
									prt('[?] List Backups', HermSetup);
									prt('[B] Buy Backup', HermSetup);
									prt('[S] Sell Backup', HermSetup);
									prt('[Q] Return to LLL', HermSetup);
									bCR(procs[0]);
									bCR(procs[0]);
									LettersPrompt('Choice - ', '?BSQ', 1, true, false, true, char(0), procs[4]);
									privates^^.stage[curNode^] := 36
								end;
							36: 	{parse buy/sell backup menu}
								if length(n[curNode^]^.curPrompt) > 0 then
									case n[curNode^]^.curPrompt[1] of
										'B': 
											privates^^.stage[curNode^] := 37;
										'S': 
											privates^^.stage[curNode^] := 41;
										'Q': 
											privates^^.stage[curNode^] := lll_menu_stage;
										'?': 
											begin
												print_text_rsrc(backup_list_txt, HermSetup);
												privates^^.stage[curNode^] := 35
											end;
										otherwise
											privates^^.stage[curNode^] := 35
									end
								else
									privates^^.stage[curNode^] := 35;
							37: 	{buy a backup}
								begin
									NumbersPrompt('Buy which backup - ', '', 15, 1, procs[5]);
									privates^^.stage[curNode^] := 38
								end;
							38: 	{""}
								with privates^^ do
									begin
										stage[curNode^] := 35;
										t1 := s2n(n[curNode^]^.curPrompt);
										if (t1 < 1) or (t1 > 15) then
											prt('There is no such backup.', HermSetup)
										else if t1 = 1 then
											prt('You can''t buy nothing - you already have it!', HermSetup)
										else
											begin
												convert_backup(t1, t2, l1, s1);
												if s1 = the_leech[curNode^].Backup then
													prt('You already own that.', HermSetup)
												else if l1 > the_leech[curNode^].Money then
													prt(not_enough_money, HermSetup)
												else
													begin
														YesNoQuestion(concat(are_you_sure, 'buy ', s1, '? '), true, procs[6]);
														stage[curNode^] := 39
													end
											end
									end;
							39: 	{""}
								with privates^^ do
									begin
										if n[curNode^]^.curPrompt[1] = 'Y' then
											begin
												the_leech[curNode^].Money := the_leech[curNode^].Money - l1;
												bCR(procs[0]);
												prt(concat('Backup "', s1, '" bought.'), HermSetup);
												convert_backup(t1, the_leech[curNode^].BackupPower, l1, the_leech[curNode^].Backup);
												print_current_money(HermSetup)
											end;
										privates^^.stage[curNode^] := 35
									end;
							40: 	{list registered leeches}
								begin
									list_leeches(HermSetup);
									privates^^.stage[curNode^] := lll_menu_stage
								end;
							41: 	{sell a backup}
								with privates^^ do
									with privates^^.the_leech[curNode^] do
										begin
											if Backup = 'None' then
												l1 := 0
											else if Backup = 'Punch Cards' then
												l1 := 100
											else if Backup = 'Paper Strips' then
												l1 := 200
											else if Backup = 'Floppy Disk' then
												l1 := 400
											else if Backup = 'High Density Floppy' then
												l1 := 800
											else if Backup = 'Tape Backup' then
												l1 := 2000
											else if Backup = 'Long Tape Backup' then
												l1 := 4000
											else if Backup = 'Elephant' then
												l1 := 8000
											else if Backup = 'Duplicate Computer' then
												l1 := 20000
											else if Backup = 'Duplicate Computer w/Lock' then
												l1 := 40000
											else if Backup = 'Federal Protection' then
												l1 := 80000
											else if Backup = 'Worldwide Protection' then
												l1 := 2000000
											else if Backup = 'Teenager to Busy Line' then
												l1 := 4000000
											else if Backup = 'Read-Only Attribute' then
												l1 := 8000000
											else if Backup = 'All Backups' then
												l1 := 20000000
											else
												l1 := 0;
											if Backup = 'None' then
												begin
													prt('You don''t have anything to sell!', HermSetup);
													stage[curNode^] := 35
												end
											else
												begin
													prt(concat('You can sell your ', Backup, ' for $', cstrl(l1 div 2), '.'), HermSetup);
													bCR(procs[0]);
													YesNoQuestion(concat(are_you_sure, 'sell it? '), false, procs[6]);
													stage[curNode^] := 42
												end
										end;
							42: 	{""}
								with privates^^.the_leech[curNode^] do
									begin
										if n[curNode^]^.curPrompt = 'Y' then
											begin
												BackupPower := 1;
												Money := Money + (privates^^.l1 div 2);
												prt(concat('Backup "', Backup, '" sold.'), HermSetup);
												Backup := 'None';
												print_current_money(HermSetup)
											end;
										privates^^.stage[curNode^] := 35
									end;
							43: 	{buy/sell software menu}
								begin
									bCR(procs[0]);
									prt('[?] List Software', HermSetup);
									prt('[B] Buy Software', HermSetup);
									prt('[S] Sell Software', HermSetup);
									prt('[Q] Return to LLL', HermSetup);
									bCR(procs[0]);
									bCR(procs[0]);
									LettersPrompt('Choice - ', '?BSQ', 1, true, false, true, char(0), procs[4]);
									privates^^.stage[curNode^] := 44
								end;
							44: 	{parse buy/sell software menu}
								if length(n[curNode^]^.curPrompt) > 0 then
									case n[curNode^]^.curPrompt[1] of
										'B': 
											privates^^.stage[curNode^] := 45;
										'S': 
											privates^^.stage[curNode^] := 48;
										'Q': 
											privates^^.stage[curNode^] := lll_menu_stage;
										'?': 
											begin
												print_text_rsrc(software_list_txt, HermSetup);
												privates^^.stage[curNode^] := 43
											end;
										otherwise
											privates^^.stage[curNode^] := 43
									end
								else
									privates^^.stage[curNode^] := 43;
							45: 	{buy a software}
								begin
									NumbersPrompt('Buy which software - ', '', 15, 1, procs[5]);
									privates^^.stage[curNode^] := 46
								end;
							46: 	{""}
								with privates^^ do
									begin
										stage[curNode^] := 43;
										t1 := s2n(n[curNode^]^.curPrompt);
										if (t1 < 1) or (t1 > 15) then
											prt('There is no such software.', HermSetup)
										else
											begin
												convert_software(t1, t2, l1, s1);
												if s1 = the_leech[curNode^].Software then
													prt('You already own that.', HermSetup)
												else if l1 > the_leech[curNode^].Money then
													prt(not_enough_money, HermSetup)
												else
													begin
														YesNoQuestion(concat(are_you_sure, 'buy ', s1, '? '), true, procs[6]);
														stage[curNode^] := 47
													end
											end
									end;
							47: 	{""}
								with privates^^ do
									begin
										if n[curNode^]^.curPrompt[1] = 'Y' then
											begin
												the_leech[curNode^].Money := the_leech[curNode^].Money - l1;
												bCR(procs[0]);
												prt(concat('Software "', s1, '" bought.'), HermSetup);
												convert_software(t1, the_leech[curNode^].SoftwarePower, l1, the_leech[curNode^].Software);
												print_current_money(HermSetup)
											end;
										privates^^.stage[curNode^] := 43
									end;
							48: 	{""}
								with privates^^ do
									with privates^^.the_leech[curNode^] do
										begin
											if Software <> 'Dumb Terminal' then
												begin
													if Software = 'Turbo-Term' then
														l1 := 100
													else if Software = 'SmartCom' then
														l1 := 200
													else if Software = 'QModem SST' then
														l1 := 400
													else if Software = 'ProComm' then
														l1 := 800
													else if Software = 'Hacker''s Delight' then
														l1 := 2000
													else if Software = 'Trojan Generator' then
														l1 := 4000
													else if Software = 'ANSI Porno Picture' then
														l1 := 8000
													else if Software = 'Dumb Comm' then
														l1 := 20000
													else if Software = 'Keyboard Locker' then
														l1 := 40000
													else if Software = 'ASCII Express' then
														l1 := 80000
													else if Software = 'ProTerm' then
														l1 := 2000000
													else if Software = 'Hermes Terminal Prog.' then
														l1 := 4000000
													else if Software = 'Super-Deleter' then
														l1 := 8000000
													else if Software = 'WipeOut Comm' then
														l1 := 20000000
													else
														l1 := 0;
													prt(concat('You can sell your ', Software, ' for $', cstrl(l1 div 2), '.'), HermSetup);
													bCR(procs[0]);
													YesNoQuestion(concat(are_you_sure, 'sell it? '), false, procs[6]);
													stage[curNode^] := 49
												end
											else
												begin
													prt('You can''t sell a dumb terminal - it''s standard!', HermSetup);
													stage[curNode^] := 43
												end
										end;
							49: 	{""}
								with privates^^.the_leech[curNode^] do
									begin
										if n[curNode^]^.curPrompt = 'Y' then
											begin
												SoftwarePower := 1;
												Money := Money + (privates^^.l1 div 2);
												prt(concat('Software "', Software, '" sold.'), HermSetup);
												Software := 'Dumb Terminal';
												print_current_money(HermSetup)
											end;
										privates^^.stage[curNode^] := 43
									end;
							50: 	{call a BBS}
								if privates^^.the_leech[curNode^].CallsToday <= 0 then
									begin
										prt('You can''t call any more BBS''s this game.', HermSetup);
										privates^^.stage[curNode^] := main_menu_stage
									end
								else
									begin
										prt(concat('Welcome to ', privates^^.the_leech[curNode^].Software, '!'), HermSetup);
										bCR(procs[0]);
										bCR(procs[0]);
										LettersPrompt('Call which BBS ("?" for list, "Q" to abort) - ', '?Q1234567890', 3, false, false, true, char(0), procs[4]);
										privates^^.stage[curNode^] := 51
									end;
							51: 	{""}
								with privates^^ do
									if n[curNode^]^.curPrompt[1] = 'Q' then
										stage[curNode^] := main_menu_stage
									else if (length(n[curNode^]^.curPrompt) <= 0) then
										stage[curNode^] := 50
									else if (n[curNode^]^.curPrompt[1] = '?') then
										begin
											print_text_rsrc(board_list_txt, HermSetup);
											stage[curNode^] := 50
										end
									else
										begin
											t1 := s2n(n[curNode^]^.curPrompt);
											if (t1 < 1) or (t1 > 99) then
												begin
													prt('There is no such bbs.', HermSetup);
													stage[curNode^] := main_menu_stage
												end
											else
												begin
													get_the_bbs(t1, bbs_name, bbs_number, HermSetup);
													OutChr(chr(12), procs[12]);
													prt(concat('Dialing "', bbs_name, '" at ', bbs_number, '...'), HermSetup);
													bCR(procs[0]);
													bCR(procs[0]);
													if round(rnd(10, HermSetup) + 1) <> 5 then
														begin
															t2 := round(rnd(7, HermSetup) + 1);
															for i := 1 to t2 do
																begin
																	prt('>>RING<<', HermSetup);
																	dly(40)
																end;
															bCR(procs[0]);
															prt('CONNECT', HermSetup);
															the_leech[curNode^].CallsToday := the_leech[curNode^].CallsToday - 1;
															the_enemy[curNode^].CurMegs := t1 * 7;
															the_enemy[curNode^].Money := t1 * 60;
															the_enemy[curNode^].BackupPower := 1;
															the_enemy[curNode^].SoftwarePower := t1;
															bCR(procs[0]);
															bCR(procs[0]);
															prt('You begin to hack!', HermSetup);
															stage[curNode^] := fight_stage
														end
													else
														begin
															prt('BZZT!  BZZT!  BZZT!', HermSetup);
															bCR(procs[0]);
															bCR(procs[0]);
															prt('Hmmm...  Looks like some other leech is after that board!', HermSetup);
															prt('Better call back later and see if he was successful!', HermSetup);
															stage[curNode^] := main_menu_stage
														end
												end
										end;
							fight_stage: 	{Battle: Begin fighting}
								with privates^^.the_leech[curNode^] do
									begin
										bCR(procs[0]);
										bCR(procs[0]);
										prt(concat('You have ', cstr(CurMegs), ' megs left.'), HermSetup);
										prt(concat('They have ', cstr(privates^^.the_enemy[curNode^].CurMegs), ' megs left.'), HermSetup);
										bCR(procs[0]);
										LettersPrompt('[T]rash files, [D]rop Carrier, [U]pload Trojan Horse, or [B]ackup Files: ', 'TDBU', 1, true, false, true, char(0), procs[4]);
										privates^^.stage[curNode^] := 53
									end;
							53: 	{Battle: Parse fighting selection}
								begin
									bCR(procs[0]);
									case n[curNode^]^.curPrompt[1] of
										'T': 
											trash_disk(HermSetup);
										'D': 
											drop_carrier(HermSetup);
										'B': 
											fast_backup(HermSetup);
										'U': 
											upload_trojan(HermSetup)
									end;
									if (n[curNode^]^.curPrompt[1] <> 'D') then
										privates^^.stage[curNode^] := 56
									else
										privates^^.stage[curNode^] := main_menu_stage	{dropped carrier; exit to main menu}
								end;
							54: 	{Battle: Leech won the game; act accordingly}
								begin
									add_to_halloffame(HermSetup);
									reset_game(HermSetup);
									privates^^.stage[curNode^] := exit_stage
								end;
							56: 	{Battle: Check everyone's statistics to see if the battle is over}
								with privates^^ do
									if (the_leech[curNode^].Level >= 31) and (the_enemy[curNode^].CurMegs <= 0) then
										begin
											print_text_rsrc(won_final_battle_txt, HermSetup);
											bCR(procs[0]);
											privates^^.pReturnStage := 54;
											stage[curNode^] := pause_stage
										end
									else if the_enemy[curNode^].CurMegs <= 0 then
										begin
											they_dead(HermSetup);
											stage[curNode^] := main_menu_stage
										end
									else if the_leech[curNode^].CurMegs <= 0 then
										begin
											you_dead(HermSetup);
											stage[curNode^] := exit_stage
										end
									else
										stage[curNode^] := fight_stage;
							60: 	{format hard drive; re-roll game}
								begin
									prt('This will start your game over as a level 1 leech.', HermSetup);
									bCR(procs[0]);
									YesNoQuestion(concat(are_you_sure, 'do this? '), false, procs[6]);
									privates^^.stage[curNode^] := 61
								end;
							61: 	{""}
								if n[curNode^]^.curPrompt[1] = 'Y' then
									begin
										format_drive(HermSetup);
										privates^^.stage[curNode^] := exit_stage
									end
								else
									privates^^.stage[curNode^] := main_menu_stage;
							62: 	{hack a leech}
								if privates^^.the_leech[curNode^].HacksToday <= 0 then
									begin
										prt('You can''t hack any more leeches this game.', HermSetup);
										privates^^.stage[curNode^] := main_menu_stage
									end
								else
									begin
										LettersPrompt('Enter the name or number of the leech to hack: ', '', 31, false, false, true, char(0), procs[4]);
										ANSIPrompter(31, procs[9]);
										privates^^.stage[curNode^] := 63
									end;
							63: 	{""}
								if FindUser(n[curNode^]^.curPrompt, the_user, procs[15]) then
									begin
										find_user(the_user.UserNum, privates^^.the_leech[curNode^].RealNumber, HermSetup, privates^^.temp_leech[curNode^]);
										if privates^^.successful then
											privates^^.stage[curNode^] := 64
										else
											begin
												prt(inval_leech, HermSetup);
												privates^^.stage[curNode^] := main_menu_stage
											end
									end
								else
									begin
										prt(inval_leech, HermSetup);
										privates^^.stage[curNode^] := main_menu_stage
									end;
							64: 	{""}
								with privates^^ do
									begin
										is_hacking := true;
										the_leech[curNode^].HacksToday := the_leech[curNode^].HacksToday - 1;
										the_enemy[curNode^].CurMegs := temp_leech[curNode^].Level * 10;
										the_enemy[curNode^].Money := temp_leech[curNode^].Money;
										the_enemy[curNode^].BackupPower := temp_leech[curNode^].BackupPower;
										the_enemy[curNode^].SoftwarePower := temp_leech[curNode^].SoftwarePower;
										bCR(procs[0]);
										bCR(procs[0]);
										prt(concat('You begin to hack "', temp_leech[curNode^].Name, '"!'), HermSetup);
										stage[curNode^] := fight_stage
									end;
							65: 
								begin
									bCR(procs[0]);
									prt('Each fast backup costs $200.', HermSetup);
									print_current_money(HermSetup);
									bCR(procs[0]);
									NumbersPrompt(concat('Enter number of fast backups you wish to buy: '), '', 163, 0, procs[5]);
									privates^^.stage[curNode^] := 66
								end;
							66: 
								with privates^^ do
									begin
										t1 := s2n(n[curNode^]^.curPrompt);
										bCR(procs[0]);
										if t1 > 0 then
											begin
												prt(concat(cstr(t1), ' fastbacks will cost you $', cstrl(t1 * 200), '.'), HermSetup);
												bCR(procs[0]);
												YesNoQuestion('Do you wish to purchase? ', true, procs[6]);
												privates^^.stage[curNode^] := 67
											end
										else
											begin
												prt('Invalid number.', HermSetup);
												privates^^.stage[curNode^] := lll_menu_stage
											end
									end;
							67: 
								with privates^^ do
									begin
										bCR(procs[0]);
										if n[curNode^]^.curPrompt[1] = 'Y' then
											if t1 * 200 > the_leech[curNode^].Money then
												prt(not_enough_money, HermSetup)
											else
												begin
													prt('Fast backups bought.', HermSetup);
													bCR(procs[0]);
													the_leech[curNode^].Money := the_leech[curNode^].Money - (t1 * 200);
													the_leech[curNode^].FastBacks := the_leech[curNode^].FastBacks + t1;
													print_current_money(HermSetup)
												end;
										stage[curNode^] := lll_menu_stage
									end;
							68: 
								begin
									bCR(procs[0]);
									prt('Each trojan horse costs $300.', HermSetup);
									print_current_money(HermSetup);
									bCR(procs[0]);
									NumbersPrompt(concat('Enter number of trojans you wish to buy: '), '', 109, 0, procs[5]);
									privates^^.stage[curNode^] := 69
								end;
							69: 
								with privates^^ do
									begin
										t1 := s2n(n[curNode^]^.curPrompt);
										bCR(procs[0]);
										if t1 > 0 then
											begin
												prt(concat(cstr(t1), ' trojans will cost you $', cstrl(t1 * 300), '.'), HermSetup);
												bCR(procs[0]);
												YesNoQuestion('Do you wish to purchase? ', true, procs[6]);
												privates^^.stage[curNode^] := 70
											end
										else
											begin
												prt('Invalid number.', HermSetup);
												privates^^.stage[curNode^] := lll_menu_stage
											end
									end;
							70: 
								with privates^^ do
									begin
										bCR(procs[0]);
										if n[curNode^]^.curPrompt[1] = 'Y' then
											if t1 * 300 > the_leech[curNode^].Money then
												prt(not_enough_money, HermSetup)
											else
												begin
													prt('Trojans bought.', HermSetup);
													bCR(procs[0]);
													the_leech[curNode^].Money := the_leech[curNode^].Money - (t1 * 300);
													the_leech[curNode^].Trojans := the_leech[curNode^].Trojans + t1;
													print_current_money(HermSetup)
												end;
										stage[curNode^] := lll_menu_stage
									end;
							pause_stage: 
								begin
									PAUSEPrompt(pause_string, procs[14]);
									n[curNode^]^.savedBDAction := none;
									privates^^.stage[curNode^] := privates^^.pReturnStage
								end;
							exit_stage: 
								begin
									if privates^^.the_leech[curNode^].Level < 31 then
										write_user(privates^^.the_leech[curNode^], HermSetup);
									bCR(procs[0]);
									bCR(procs[0]);
									prt('Now returning to an environment with even WORSE users...', HermSetup);
									bCR(procs[0]);
									privates^^.activeOn[curNode^] := false;
									privates^^.stage[curNode^] := 0;
									n[curNode^]^.activeuserExternal := -1
								end;
							otherwise
						end;
					otherwise
				end
			end
	end;

end.